@@ -260,6 +260,7 @@ def default_v2(g,nk=400):
     fbl.write(str(2*nk/3)+"   K'\n")
     fbl.write(str(nk)+"   \Gamma\n")
     fbl.close()
+  else: raise # not implemented
 
 
 
@@ -287,3 +288,32 @@ def kmesh(dimensionality,nk=10,nsuper=1):
 
 
 
+
+def label2k(g,kl):
+    """Given a kpoint label, return the kpoint"""
+    if kl=="G": return [0.,0.,0.]
+    elif kl=="M": return [0.5,0.,0.]
+    elif kl=="K":
+        from .kpointstk.locate import target_moduli
+        from .kpointstk.mapping import unitary
+        vm = np.sqrt(3) # moduli
+        k = g.k2K(target_moduli(unitary(g.b1),unitary(g.b2),vm))
+        return k*2/(3.*np.sqrt(3))
+    elif kl=="K'": return -label2k(g,"K")
+    elif kl=="M1": return [.5,.0,.0]
+    elif kl=="M2": return [.0,.5,.0]
+    elif kl=="M3": return [.5,.5,.0]
+    else: raise
+
+
+
+
+
+def get_kpath(g,ks,**kwargs):
+    """Return the k-path"""
+    kps = [label2k(g,k) for k in ks] # get the kpoints
+    from .kpointstk.locate import k2path
+    return k2path(g,kps,**kwargs) # closest path
+
+
+
